/****************************************************************
 * wavemath.c
 * Last change 12-22-2013 by Osman Celimli
 ****************************************************************/
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>

#include <stdio.h>
#include <string.h>
#include <windows.h>

#include "wavemath.h"
/****************************************************************
 * int wave_adjScale(float *pafSignal, unsigned int uiLen,
 *	float fNewMin, float fNewMax)
 * Adjust the offset and amplitude of the given signal such that
 * all of its samples fall between the given min and max. Yields
 * nonzero if any errors were encountered during processing.
 ****************************************************************/
int wave_adjScale(float *pafSignal, unsigned int uiLen,
	float fNewMin, float fNewMax)
{
	unsigned int uiX,uiY;
	float fMax,fMin;

	float fOffset,fScalar;

	/**
	 * First get the original maximum and minimum of the waveform,
	 * this will also handle all range and NULL checks.
	 */
	if(wave_getPeaks(pafSignal, uiLen, &fMax, &fMin, &uiX, &uiY))
	{
		return -1;
	}

	/**
	 * Each sample is adjusted such that:
	 *	New Amplitude = (Old Amplitude + Offset) * Scalar
	 * Where the offset is the difference between the target and
	 * original minimums and the scalar is the ratio of the target
	 * amplitude swing (max - min) over the original.
	 *
	 * This should also allow for a waveform to be inverted by
	 * feeding this routine with a target maximum which is less
	 * than the target minimum.
	 */
	fOffset = fNewMin - fMin;
	fScalar = (fNewMax - fNewMin) / (fMax - fMin);

	for(uiX = 0; uiX < uiLen; uiX++)
	{
		pafSignal[uiX] = (pafSignal[uiX] + fOffset) * fScalar;
	}
	return 0;
}




/****************************************************************
 * int wave_findVal(float *pafSignal, unsigned int uiLen,
 *	float fTarget, float fThresh, unsigned int *puiOffset)
 * Attempt to locate the value fTarget +/- fThresh within
 * the supplied signal. If the value is found its index will be
 * written to *puiOffset and a nonzero value will be returned.
 ****************************************************************/
int wave_findVal(float *pafSignal, unsigned int uiLen,
	float fTarget, float fThresh, unsigned int *puiOffset)
{
	unsigned int uiX;

	if((pafSignal == NULL) || (puiOffset == NULL)) return 0;

	for(uiX = 0; uiX < uiLen; uiX++)
	{
		if(((pafSignal[uiX] - fThresh) < fTarget) &&
			((pafSignal[uiX] + fThresh) > fTarget))
		{
			*puiOffset = uiX;
			return -1;
		}
	}
	return 0;
}
/****************************************************************
 * int wave_getPeaks(float *pafSignal, unsigned int uiLen,
 *	float *pfMax, float *pfMin,
 *	unsigned int *puiMaxX, unsigned int *puiMinX)
 * Find the maximum and minimum of the given signal, returns
 * nonzero if any errors were encountered during the operation.
 ****************************************************************/
int wave_getPeaks(float *pafSignal, unsigned int uiLen,
	float *pfMax, float *pfMin,
	unsigned int *puiMaxX, unsigned int *puiMinX)
{
	float fCurMax,fCurMin;
	unsigned int uiX,uiY,uiZ;

	if((pafSignal == NULL) || (pfMax == NULL) || (pfMin == NULL)
		|| (puiMaxX == NULL) || (puiMinX == NULL) || (uiLen < 1))
	{
		return -1;
	}

	fCurMax = pafSignal[0];
	fCurMin = pafSignal[0];
	uiY = 0;
	uiZ = 0;
	for(uiX = 1; uiX < uiLen; uiX++)
	{
		if(pafSignal[uiX] > fCurMax)
		{
			fCurMax = pafSignal[uiX];
			uiY = uiX;
		}
		if(pafSignal[uiX] < fCurMin)
		{
			fCurMin = pafSignal[uiX];
			uiZ = uiX;
		}
	}

	*pfMax = fCurMax;
	*puiMaxX = uiY;
	*pfMin = fCurMin;
	*puiMinX = uiZ;
	return 0;
}
